#include <boost/random.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <iostream>
#include <string>
#include <set>

#include "PtDebug.h"
#include "time.h"
#include "utility.h"
#include "PtThread.h"

using namespace boost;

int main(int argc, char ** argv)
{
  if(argc != 5)
    {
      cout << "Input:" << endl <<
	"1. graph path" << endl <<
	"2. T " << endl <<
	"3. K" << endl <<
	"4. theta" << endl;

      return -1;
    }

  ifstream is(argv[1]);

  if(!is)
    {
      Pt_Info << "open file error... " << argv[1] << endl;
      return -1;
    }

  vector<list<int> > g;
  ReadSnapGraph(g, is);

  vector<list<RawLink> > rawGraph;
  vector<double> actProb, latProb;
  int K, T;
  double theta;

  for(int i = 0; i < 20; ++i)
    latProb.push_back(i + 1);


  T = atoi(argv[2]);
  K = atoi(argv[3]);
  theta = atof(argv[4]);

  ConvertToWCRawGraph(g, rawGraph, latProb);

  vector<list<int> >().swap(g);
  set<int> seed;

  for(int i = 0; i < argc; ++i)
    cout << argv[i] << " ";
  cout << endl;

  Pt_Info << "IPCelfKNodes() starting ..." << endl;
  IPCelfKNodes(T, rawGraph, theta, K, seed);
  Pt_Info << "IPCelfKNodes() ending ..." << endl;

  Pt_Info << "influence for IP : " << MultiMonteCarlo(20000, T, seed, rawGraph) << endl << "seed set:" << endl;
  for(set<int>::iterator itor = seed.begin();
      itor != seed.end();
      ++itor)
    {
      cout << *itor << " ";
    }
  cout << endl;

  Pt_Info << "Peak memory : " << GetPeakMem() << " MB" << endl;

}

