#ifndef __UTILITY_H__
#define __UTILITY_H__


#include <set>
#include <fstream>
#include <vector>
#include <list>

using namespace std;


struct PathWithSet
{
  int endNode;
  double weight;
  int length;
  set<int> usedNodes;
};

struct Path
{
  int endNode;
  double weight;
  //  int length;
};


struct Link
{
  int node; // node of the other end
  double weight; // the probability of this link
  //  int length; // the time length of this link
};


struct RawLink
{
  int node; // node of the other end
  double ap; // activating probability
  //  double lambda;// influence delay distribution, poisson with parameter lambda
};

void ReadFlixsterGraph(vector<list<int> > & g, ifstream & is);

void ReadSnapGraph(vector<list<int> > & g, ifstream & is);



void ConvertToTVGraph(const vector<list<int> > & g, vector<list<Link> > & tvGraph, int T, vector<double> & actProb, vector<double> & latProb);



// convert a plain graph to raw asIC graph
void ConvertToRawGraph(const vector<list<int> > & g, vector<list<RawLink> > & rawGraph, vector<double> & actProb, vector<double> & latProb);


// convert a plain graph to raw graph with WC model (probabiilty of activating node v is equal to 1 / indegree(v))
void ConvertToWCRawGraph(const vector<list<int> > & g, vector<list<RawLink> > & rawGraph);


void AP(int T, const set<int> & seed, vector<vector<double> > & ap, const vector<list<Link> > & graph);


double DeltaInflu(int T, const vector<int> & seed, int u, const vector<list<Link> > & graph);


// calculate the influence of seed 
int MonteCarlo(const set<int> & seed, const vector<list<RawLink> > & graph);

// calculate the average influence R times
double MultiMonteCarlo(int R, const set<int> & seed, const vector<list<RawLink> > & graph);


// find all simple paths starting from a node in seed, with length no larger than T, and weight no less the theta
// the path can not contain any nodes in seed except the starting node, and must ending at a node not in seed
void FindPath(const set<int> & seed, const vector<list<RawLink> > & graph, double theta, vector<Path> & paths);


double CalInflu(const vector<Path> & paths);

// return the influence caused by seed with independant path method
//double IPInflu(int T, const set<int> & seed, const vector<list<RawLink> > & graph, double theta, vector<Path> & paths);
double IPInflu(const set<int> & seed, const vector<list<RawLink> > & graph, double theta);

// return the K most influential nodes with independant path method
void IPCelfKNodes(const vector<list<RawLink> > & graph, double theta, int K, set<int> & kNodes);


// return the K most influential nodes with degree ratio combined independant path method
// the performance of this function if very bad ...
void RatioCombIPCelfKNodes(int T, const vector<list<RawLink> > & graph, double theta, int K, set<int> & kNodes);

// return the K most influential nodes with degree weighted ratio combined independant path method
void RatioWeightedCombIPCelfKNodes(const vector<list<RawLink> > & graph, double theta, int K, set<int> & kNodes);

// return the K most influential nodes with degree weighted ratio combined independent path method by using "threadNum" number of threads
void RatioWeightedCombIPCelfKNodesMT(const vector<list<RawLink> > & graph, double theta, int K, set<int> & kNodes, int threadNum);

void IPCelfKNodesMT(const vector<list<RawLink> > & graph, double theta, int K, set<int> & kNodes, int threadNum);

// return the K most influential nodes with path nodes set combined independant path method
void SetCombIPCelfKNodes(int T, const vector<list<RawLink> > & graph, double theta, int K, set<int> & kNodes);


// return the K most influential nodes with monte carlo
void MCCelfKNodes(const vector<list<RawLink> > & graph, int K, set<int> & kNodes, int R);

// return the K nodes with largest out degree
void LargeDegreeKNodes(const vector<list<RawLink> > & graph, int K, set<int> & kNodes);

// randomly return K nodes
void RandomKNodes(const vector<list<RawLink> > & graph, int K, set<int> & kNodes);

// return the K nodes with largest IP influence
void LargeIPKNodes(int T, const vector<list<RawLink> > & graph, double theta, int K, set<int> & kNodes);

// return the peak memory usage of the calling process in MB
// negtive value is returned when error
double GetPeakMem(void);


#endif
